public class Test {
	public static void main(String[] args) {
		System.out.println("Hello, World!");
		// Comment
		int j = 20;
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}

		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
		for(int i = 0; i < j; i++) {
			double x = 5.0;
			System.out.println(j/x);
			if (i == 5) {
				break;
			}
		}
	}
}
