public class Test2 {
	public static void main(String[] args) {
		// comment 1
		System.out.println("Hello, World!");
		String z = "test";
		for(var i = 0; i <= 30; i++) {
			System.out.println(z);
		}
		// comment 2
	}
}
