#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

int main(void) {
    int *ptr = malloc(sizeof(int) * 20);
    printf("Hello World\n");
    return 0;
}
